package net.grcomputing.opencms.search.lucene;

/*
* <p>
* This file is part of GRC's OpenCMS module library.
* </p>
* <p>
* GRC's OpenCMS module library 
* is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
* </p>
* <p>
* GRC's OpenCMS module library is distributed in the 
* hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* </p>
* <p>
* You should have received a copy of the GNU General Public License
* along with GRC's OpenCMS module library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
* Or view the license online at http://www.gnu.org/licenses/gpl.html
* </p>
* <hr>
*/
import java.util.HashMap;

import org.apache.lucene.document.DateField;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;

import com.opencms.core.CmsException;
import com.opencms.file.CmsFile;
import com.opencms.file.CmsObject;

/**
* @author Matt Butcher mbutcher@grcomputing.net
* @see <a href="http://grcomputing.net">http://grcomputing.net</a>
* <p>This class serves as a document factory for OpenCMS resources. It produces
* Lucene Document objects that contain the correct fields for indexing OpenCMS
* resources. Unlike some of the other Lucene implementations, this one is
* highly coupled with the OpenCMS API - thereby taking advantage of properties
* security settings, etc.</p>
* <p>Essentially, the main purpose of this method is to provide factories for
* Document instances that are specific to the types of resources in OpenCMS.
* For instance, we know that a Page will have a name, a title, and probably
* description and keywords. We also know that it is an XML document with HTML
* stored in the CDATA section. These bits of information provide information
* that we can use to generate an appropriate Document representation.</p>
*/
public class BodylessDocument implements I_DocumentConstants, I_DocumentFactory 
	{
	
	/**
	 * Takes an instance of some type and builds a Lucene Document suitable for 
	 * index generation. This factory will <i>not</i> attempt to include any
	 * information about the body of the CmsFile. For that reason, it is
	 * a good candidate for JSP or XML file indexing. Think of this as a 
	 * metadata-only factory.
	 *
	 * @param CmsObject cms - instance of CmsObject
	 * @param CmsFile f - file that we will generate Document from.
	 * @throws CmsException it cannot work with the CmsFile or CmsObject.
	 */
	public Document Document(CmsObject cmso, CmsFile f, HashMap p)
		throws CmsException 
	{
		Document doc = new Document();
		String absPath = f.getAbsolutePath();

		// This is location in VFS -- not URL.
		doc.add(Field.UnIndexed(FIELD_PATH, absPath));

		// Get date info. I hope the long is actually a millisecond string...
		doc.add(Field.Keyword(FIELD_INITIAL_ADD, 
			DateField.timeToString(f.getDateCreated())));

		doc.add(Field.Keyword(FIELD_LAST_MOD, 
			DateField.timeToString(f.getDateLastModified())));

		// Check for properties:
		String title, description, keywords;
	
		if((title = cmso.readProperty(absPath, "title")) != null) {
			doc.add(Field.Keyword(FIELD_TITLE, title));
		}
		if((description = cmso.readProperty(absPath, "description")) != null) {
			doc.add(Field.Keyword(FIELD_DESC, description));
		}
		if((keywords = cmso.readProperty(absPath, "keywords")) != null) {
			doc.add(Field.Keyword(FIELD_KEYWORDS, keywords));
		}
		if(p!=null && (!(p.get("section")==null))) 
		{
			doc.add(Field.Keyword(FIELD_SECTION, (String)p.get("section")));
		}
		/*
		 * That's it.
		 */
		return doc;
	}

	/**
	 * Takes an instance of some type and builds a Lucene Document suitable for 
	 * index generation. This factory will <i>not</i> attempt to include any
	 * information about the body of the CmsFile. For that reason, it is
	 * a good candidate for JSP or XML file indexing. Think of this as a 
	 * metadata-only factory.
	 *
	 * @param CmsObject cms - instance of CmsObject
	 * @param CmsFile f - file that we will generate Document from.
	 * @param HashMap params - unused.
	 * @throws CmsException it cannot work with the CmsFile or CmsObject.
	 */
	public Document Document(CmsObject cmso, CmsFile f)
		throws CmsException 
	{
		return Document(cmso,f, null);
	}
}

