package net.grcomputing.opencms.search.lucene;

/*
* <p>
* This file is part of GRC's OpenCMS module library.
* </p>
* <p>
* GRC's OpenCMS module library 
* is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
* </p>
* <p>
* GRC's OpenCMS module library is distributed in the 
* hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* </p>
* <p>
* You should have received a copy of the GNU General Public License
* along with GRC's OpenCMS module library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
* Or view the license online at http://www.gnu.org/licenses/gpl.html
* </p>
* <hr>
* 
*/

import java.util.Date;
import java.util.HashMap;

import com.opencms.core.I_CmsCronJob;
import com.opencms.file.CmsObject;

/**
* Handles the cron updates to the Lucene index.
* @author Matt Butcher mbutcher@grcomputing.net
* @see <a href="http://grcomputing.net">http://grcomputing.net</a>
* 
*/
public class CronIndexManager implements I_CmsCronJob {

	/** Param to control whether or not index is created.
	 * IF this param is set to "true", then the old index will be
	 * destroyed and a new one created. Otherwise, the old index will
	 * be updated.
	 * usage in cron: "createIndex=true".
	 */
	public static String PARAM_CREATE_INDEX = "createIndex";

	/**
	 * THis param designates an alternate registry path.
	 * If this is set, then the specified registry file will be used
	 * instead of the system registry.xml file.
	 * Usage in cron: "registry=/path/to/registry.xml"
	 */
	public static String PARAM_REGISTRY_PATH = "registry";

	/**
	 * This method is executed by the cron manager. It invokes the 
	 * main IndexManager to update the Lucene indices.
	 * Indexing is a big job, and
	 * it may throw IOExceptions or CmsExceptions, and possibly classloader
	 * exceptions. Usually, and attempt is made to minimize small errors in
	 * order to get the index built.
	 *<p>Supported Params:</p><ul>
	 * <li>registry=/path/to/registry.xml</li>
	 * <li>createIndex=true</li>
	 * </ul>
	 * <p>example params string: "param1=true,param2=/path,param3=test me"</p>
	 * @see net.grcomputing.opencms.search.lucene.IndexManager
	 * @param CmsObject cms object.
	 * @param String parms in comma separated list.
	 * @throws Exception when anything goes wrong. 
	 */
	public String launch(CmsObject cmso, String params) 
		throws Exception
	{
		// By default, update index
		boolean createIndex = false;

		// By default, use system registry
		boolean useParamsRegistry = false;
		String altRegistry = "";

		HashMap args = parseParams(params);
		
		if("true".equals((String)args.get(PARAM_CREATE_INDEX)))
			createIndex = true;

		if(args.containsKey(PARAM_REGISTRY_PATH)) {
			altRegistry = (String)args.get(PARAM_REGISTRY_PATH);
			if(altRegistry != null && !"".equals(altRegistry)) 
				useParamsRegistry = true;
		}


		IndexManager im = null;
		if(useParamsRegistry) 
			im = new IndexManager(cmso, altRegistry);
		else 
			im = new IndexManager(cmso);
		
		im.doIndex(createIndex);
	
		Date now = new Date();	
		return "CronIndexManager rebuilt the Lucene index on "+now.toString();
	}

	/**
	 * Parses param string, an places each name/value pair into a HashMap.
	 * If params is null or "", returns an empty HashMap.
	 */
	private static HashMap parseParams(String params) {
		if(params == null || "".equals(params)) return new HashMap();
		String sep1 = ","; // Separates params
		String sep2 = "="; // Separates name/value
		HashMap results = new HashMap();
		String [] args = params.split(sep1, 0);
		String [] nameVal;
		int i, j = args.length;
		for (i = 0; i < j; ++i) {
			if(args[i].length() > 2) {
				nameVal = args[i].split(sep2, 2);
				if(nameVal[0].length() >= 1)
					results.put(nameVal[0], nameVal[1]);
			}
		}
		return results;
	}
}

