package net.grcomputing.opencms.search.lucene;

/*
* <p>
* This file is part of GRC's OpenCMS module library.
* </p>
* <p>
* GRC's OpenCMS module library 
* is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
* </p>
* <p>
* GRC's OpenCMS module library is distributed in the 
* hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* </p>
* <p>
* You should have received a copy of the GNU General Public License
* along with GRC's OpenCMS module library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
* Or view the license online at http://www.gnu.org/licenses/gpl.html
* </p>
* <hr>
*/

import java.util.ArrayList;

import com.opencms.core.CmsException;

/**
* <p>This class maps extensions to Lucene search document factories (e.g.
* implementations of I_DocumentFactory).</p>
* <p>Extensions are used by the indexer to determine what document factory
* ought to be used to index the content of particular resources. Normally,
* extensions are only checked for content of type "plain", not Pages or JSPs.
* </p>
* <p>I have tried to set suitable default values, such as a default document
* factory of net.grcomputing.opencms.search.lucene.BodylessDocument.</p>
* @author Matt Butcher mbutcher@grcomputing.net
* @see <a href="http://grcomputing.net">http://grcomputing.net</a>
*/
public class ExtensionMapping {
	private ArrayList extensions = new ArrayList();
	private String extMapName = "Default Map";

	// This is a safe default
	private String docFactory = 
		"net.grcomputing.opencms.search.lucene.BodylessDocument";

	private I_DocumentFactory df;

	/*
	 * Creates a new Extension Mapping. This creates a full ExtensionMapping
	 * using the extensions in the ArrayList and the class in docFac.
	 */
	/*
	public ExtensionMapping(ArrayList exts, String docFac, String name) {
		if(exts != null) this.extensions = exts;
		this.docFactory = docFac;
		this.extMapName = name;
		return
	}
	*/

	/**
	 * Creates a new Extension Mapping with an empty list of extensions.
	 */
	public ExtensionMapping(String docFac, String name) {
		this.docFactory = docFac;
		this.extMapName = name;
	}

	/**
	 * Adds an extension. Extensions should be of type ".xxx". If the dot is not
	 * present, it will be prepended. There is no limit on the characters,
	 * however, the underlying filesystems might have problems with non-ASCII
	 * letters. <b>NOTE: using additional dots in the extension may also cause
	 * problems with the underlying implementation of OpenCMS. The extension
	 * ".tar.gz" will likely be rendered ".gz" by OpenCMS. Therefore, it would
	 * not match the class to handle .tar.gz files here.</b>
	 *
	 * @param String extension of the form ".XXX"
	 * @return true if extension was added, false if not (which means the 
	 * extension is already there.
	 */
	public boolean addExtension(String ext) {
		// first Char should be dot
		if(ext.indexOf(".") != 0) ext = "." + ext;

		/*
		if(ext.lastIndexOf(".")
			throw new CmsException("Format error on extension " +ext +
				": Too many dots.");
		*/
		if(extensions.indexOf(ext) == -1) {
			extensions.add(ext);
			return true;
		}
		return false;
	}

	/**
	 * Removes an extension from the list of extensions. If the extension
	 * doesn't exist in the list, the return will be false. Otherwise, true.
	 * @param String extension following the rules documented for addExtension.
	 * @return true if extension was removed, false if no match was found.
	 */
	public boolean removeExtension(String ext) {
		// first Char should be dot
		if(ext.indexOf(".") != 0) ext = "." + ext;

		int i = extensions.indexOf(ext);
		if (i != -1) {
			extensions.remove(i);
		}
		return false;
	}

	/**
	 * Returns true if this ExtensionMapping handles the given extension.
	 * When this is true, it means that the DocumentFactory encapsulated in
	 * this ExtensionMap can be applied to files with the given extension.
	 * @param String extension (.XXX)
	 * @return true if this ExtensionMapping handles the given extension. False
	 * otherwise.
	 */
	public boolean handlesExtension(String ext) {
		// first Char should be dot
		if(ext.indexOf(".") != 0) ext = "." + ext;

		if (extensions.indexOf(ext) != -1) return true;
		return false;
	}

	/**
	 * Sets the name of this extension map. Mainly, names are used
	 * for debug information. it's a good idea to set this.
	 * @param String name - name of this map, e.g. "Text files"
	 */
	public void setName(String name) {
		this.extMapName = name;
		return;
	}

	// This would allow the factory to be changed after being instantiated,
	// which would cause weird errors.
	/*
	 * Sets the document factory (implementation of I_DocumentFactory).
	 * @param String full class name.
	 */
	/*
	public void setDocumentFactoryName(String docFac) {
		this.docFactory = docFac;
	}
	*/

	/**
	 * Gets the name of this extension map.
	 * @return String name
	 */
	public String getName() {
		return extMapName;
	}

	/**
	 * Gets the class name for the document factory.
	 * @return String name
	 */
	public String getDocumentFactoryName() {
		return docFactory;
	}

	/**
	 * Returns an instance of the appropriate DocumentFactory.
	 * @return An implementation of an I_DocumentFactory
	 * @throws CmsException if it can't load the class.
	 */
	public I_DocumentFactory getDocumentFactory() throws CmsException {
		if (docFactory == null) 
			throw new CmsException("Cannot load DocumentFactory null!");
	
		if(df == null) {
			try {
				Class docFactoryClass = Class.forName(docFactory);
				df = (I_DocumentFactory)docFactoryClass.newInstance();
			} catch (ClassNotFoundException cnfe) {
				df = null;
				throw new CmsException("Cannot locate DocumentFactory "
					+ docFactory, cnfe);
			} catch (InstantiationException ie) {
				df = null;
				throw new CmsException("Cannot instantiate DocumentFactory "
					+ docFactory, ie);
			} catch (IllegalAccessException iae) {
				df = null;
				throw new CmsException("Access violation: DocumentFactory "
					+ docFactory, iae);
			}
		}
		return df;
	}
}


