package net.grcomputing.opencms.search.lucene;

/*
* @author Matt Butcher mbutcher@grcomputing.net
* @see <a href="http://grcomputing.net">http://grcomputing.net</a>
* <p>
* This file is part of GRC's OpenCMS module library.
* </p>
* <p>
* GRC's OpenCMS module library 
* is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
* </p>
* <p>
* GRC's OpenCMS module library is distributed in the 
* hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* </p>
* <p>
* You should have received a copy of the GNU General Public License
* along with GRC's OpenCMS module library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
* Or view the license online at http://www.gnu.org/licenses/gpl.html
* </p>
* <hr>
* <p>This class manages the main search index for the site.</p>
*/
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.index.IndexWriter;

import com.opencms.core.A_OpenCms;
import com.opencms.core.CmsException;
import com.opencms.file.CmsFile;
import com.opencms.file.CmsFolder;
import com.opencms.file.CmsObject;
import com.opencms.file.CmsResource;

/**
* This class builds and maintains the Lucene index.
* @author Matt Butcher mbutcher@grcomputing.net
* @see <a href="http://grcomputing.net">http://grcomputing.net</a>
*/
public class  IndexManager implements I_XmlConfigConstants {
	/** Defines the name of what it considers to be a page.
	 * The way OpenCMS was constructed, there are not constants 
	 * defined for each type (since new types can be added). Search,
	 * however, needs to know a lot about every type that it is supposed
	 * to index. In the long run, I will probably make this more configurable
	 * in the future, but for now, I need something quick-n-dirty.
	 */
	public static String INDEX_TYPE_PAGE = "page";

	/** Defines the name of what it considers to be a plain resource. */
	public static String INDEX_TYPE_PLAIN = "plain";

	/** Defines the name of what it considers to be a jsp. */
	public static String INDEX_TYPE_JSP = "jsp";

	/** Defines the name of what it considers to be an XML template. */
	public static String INDEX_TYPE_XML_TEMPLATE = "XML Template";

	private static int DEFAULT_DOC_COUNT = 100000;
	private static String DEFAULT_INDEX_LOCATION = "/opt/lucene/index/opencms";
	
	/** 
	 * When the exception catcher for the processDir reaches this many
	 * exceptions, it will start throwing CmsExceptions. Until then, it will
	 * ignore them, not stopping the index creation b/c of small issues.
	 */
	private static int MAX_DOCUMENT_READING_EXCEPTION = 10;
	private int failedFiles = 0;

	private CmsObject cmso;
	private SearchParameters sParams;

	/** Do not use */
	private IndexManager () {}

	/**
	 * Constructs an IndexManager from a CmsObject. This constructor will use
	 * the default registry (usually 
	 * $TOMCAT/webapps/opencms/WEB_INF/config/registry.xml). This is preferred.
	 * @param CmsObject main CMS object
	 * @throws CmsException if it can't get the registry.
	 */
	public IndexManager(CmsObject cmso) throws CmsException {
		this.cmso = cmso;
		this.sParams = new SearchParameters(cmso);
		return;
	}

	/**
	 * Creates an index manager with a specific registry. This constuctor
	 * will use the specified registry file instead of the default one. 
	 * Registry must exist in the system's filesystem (e.g. not in VFS).
	 *
	 * @param CmsObject main CMS object
	 * @param String registryPath - path to alternate registry file
	 * @throws CmsException if it can't get the registry.
	 */
	public IndexManager(CmsObject cmso, String registryPath) 
		throws CmsException 
	{
		this.cmso = cmso;
		this.sParams = new SearchParameters(registryPath);
		return;
	}

	/**
	 * Creates an index manager with a specific registry. This constuctor
	 * will use DOM element instead of the default one. The DOM element is
	 * equivalent to the luceneSearch element in the registry file.
	 *
	 * @param CmsObject main CMS object
	 * @param org.w3c.dom.Element registryElement - element equivalent to luceneSearch
	 * @throws CmsException if it can't get the registry.
	 */
	public IndexManager(CmsObject cmso, org.w3c.dom.Element registryEle) 
		throws CmsException 
	{
		this.cmso = cmso;
		this.sParams = new SearchParameters(registryEle);
		return;
	}

	/*
	 * Creates an index manager with a specific registry -- one that may 
	 * be in the VFS. If isInVfs is true, then the constructor will read
	 * the registry from a file in the VFS. This allows the search config
	 * to be kept under CMS control.
	 * @param CmsObject main CMS object
	 * @param String registryPath - path to alternate registry file
	 * @param boolean isInVfs - whether or not to look in VFS for file.
	 * @throws CmsException if it can't get the registry.
	 */
	/*
	public IndexManager(CmsObject cmso, String registryPath, 
		boolean isInVfs) 
		throws CmsException
	{
		this.cmso = cmso;
		if(! isInVfs) {
			registry = new CmsRegistry(registryPath);
		} else {
			// FIXME
			throw new CmsException("VFS version is not implemented!");	
		}
	}
	*/

	/**
	 * Runs an IndexWriter over the appropriate files.
	 * If 'create' is true, then it will try and create the index from scratch.
	 * Otherwise it will use the existing indices.
	 * @param boolean true to create a new index.
	 * @throws CmsException if files can't be generated from CMS.
	 * @throws IOException if the Lucene engine has problems.
	 */
	public void doIndex(boolean create)
		throws CmsException, java.io.IOException
	{
		// FIXME: This needs to load the analyzer specified in the config file.
		Analyzer nlyz;
		String anaName = sParams.getAnalyzerName();

		A_OpenCms.log(A_OpenCms.C_OPENCMS_INFO, 
"=====IndexManager============================================================="
);
		// Use a default if none is specified.
		if(anaName == null) {
			nlyz = new StandardAnalyzer();
			A_OpenCms.log(A_OpenCms.C_OPENCMS_INFO, 
				"Analyzer: StandardAnalyzer");
		} else {
			A_OpenCms.log(A_OpenCms.C_OPENCMS_INFO, 
				"Analyzer: " + anaName);
			try {
				Class lx = Class.forName(anaName);
				nlyz = (Analyzer)lx.newInstance();
			} catch (Exception e) {
				throw new CmsException(
					"Could not create analyzer with classname "+
					anaName, e);
			}
		}

		//Analyzer nlyz = new StandardAnalyzer();
		//nlyz = new StandardAnalyzer();
		IndexWriter writer = new IndexWriter(sParams.getIndexPath(),
			nlyz, create);

		/*
		 * Print out some info on what DocumentFactories can be used.
		 */
		if(sParams.isUsingPageDocumentFactory())
			A_OpenCms.log(A_OpenCms.C_OPENCMS_INFO, 
				"Page DocumentFactory loaded");
		if(sParams.isUsingJspDocumentFactory())
			A_OpenCms.log(A_OpenCms.C_OPENCMS_INFO, 
				"JSP DocumentFactory loaded");
		if(sParams.isUsingXmlTemplateDocumentFactory())
			A_OpenCms.log(A_OpenCms.C_OPENCMS_INFO, 
				"XML Template DocumentFactory loaded");
		if(sParams.isUsingPlainDocumentFactory()) {
			A_OpenCms.log(A_OpenCms.C_OPENCMS_INFO, 
				"Plain DocumentFactory loaded");

			Iterator it = sParams.getPlainExtensionMappings().iterator();
			ExtensionMapping map;
			while(it.hasNext()) {
				map = (ExtensionMapping)it.next();
				A_OpenCms.log(A_OpenCms.C_OPENCMS_INFO, 
					"Extension map exists to handle " + map.getName());
			}
		}
			

		/*=====================================* 
 		  Start working on the directory reader!
 		 *=====================================*/
		HashMap dirs = sParams.getDirectories();
		Iterator i = dirs.keySet().iterator();
		String dPath, dSubsearch, section;
		boolean traverse;
		while(i.hasNext()) {
			dPath = (String)i.next();
			Map dirParams = (Map)dirs.get(dPath);
			dSubsearch = (String)dirParams.get(ELE_SUBSEARCH);
			section = (String)dirParams.get(ELE_SECTION);
			if(dSubsearch == null) traverse = sParams.needsSubsearch();
			else traverse = "true".equals(dSubsearch);

			processDir(dPath, traverse, writer, section);
		}

		String nDocs = Integer.toString(writer.docCount());
		A_OpenCms.log(A_OpenCms.C_OPENCMS_INFO, 
			"IndexManager: " + nDocs + " documents are being processed");
		
		//Document d = new Document();
		//writer.addDocument(d);
 		/*=====================================*/
		writer.close();
		A_OpenCms.log(A_OpenCms.C_OPENCMS_INFO, "Done\n"+
"=====IndexManager============================================================="
);
	}

	/**
	 * Do all the hard stuff. Essentially, this is a depth-first traversal
	 * of the directories passed in from the search config. Because the
	 * tree can't be very deep, a depth-first traversal should be fairly
	 * fast.
	 *
	 * Ideally, there should be a way to specify depth on directory
	 * scanning. Maybe I'll add that later.
	 */
	protected void processDir(String dir, boolean traverseDir, 
		IndexWriter writer, String section)
		throws CmsException
	{
		// Log the dir.
		A_OpenCms.log(A_OpenCms.C_OPENCMS_INFO, 
			"IndexManager: indexing " + dir);
		// Get files
		CmsResource res = cmso.readFileHeader(dir);
		if(res.isFolder()) {
			Vector files = cmso.getFilesInFolder(dir); 
			Enumeration e = files.elements();
			CmsFile f;
			// process files
			while(e.hasMoreElements()) {
				f = (CmsFile)e.nextElement();
				/*
				 * We don't want to stop the entire index b/c of one or two
				 * exceptions. 
				 */
				try {
					processFile(f, writer, section);
				} catch (java.io.IOException ioe) {
					if(failedFiles >= MAX_DOCUMENT_READING_EXCEPTION) {
						throw new CmsException("Failed too many file reads. "+
							"Reporting last failure.", ioe);
					} else {
						A_OpenCms.log(A_OpenCms.C_OPENCMS_CRITICAL,
							"IndexManager: " +
							"IO Error processing file " + f.getName() + ": " +
							ioe.toString());
					}
					++failedFiles;
				} catch (CmsException cmse) {
					if(failedFiles >= MAX_DOCUMENT_READING_EXCEPTION) {
						throw new CmsException("Failed too many file reads. "+
							"Reporting last failure.", cmse);
					} else {
						A_OpenCms.log(A_OpenCms.C_OPENCMS_CRITICAL,
							"IndexManager: " +
							"CMS Error processing file " + f.getName() + ": " +
							cmse.toString());
					}
					++failedFiles;
				}
			}
			// get subdirs
			if(traverseDir) {
				Vector folders = cmso.getSubFolders(dir);
				CmsFolder mfolder;
				e = folders.elements();
				// recurse with subdirs
				while (e.hasMoreElements()) {
					mfolder = (CmsFolder)e.nextElement();
					processDir(mfolder.getAbsolutePath(), traverseDir, writer, section);
				}
			}
		} else if (res.isFile()) {
			try {
				processFile(dir, writer, section);
			} catch (java.io.IOException ioe) {
				if(failedFiles >= MAX_DOCUMENT_READING_EXCEPTION) {
					throw new CmsException("Failed too many file reads. "+
						"Reporting last failure.", ioe);
				} else {
					A_OpenCms.log(A_OpenCms.C_OPENCMS_CRITICAL,
						"IndexManager: " +
						"IO(2) Error processing file " + dir + ": " +
						ioe.toString());
				}
				++failedFiles;
			} catch (CmsException cmse) {
				if(failedFiles >= MAX_DOCUMENT_READING_EXCEPTION) {
					throw new CmsException("Failed too many file reads. "+
						"Reporting last failure.", cmse);
				} else {
					A_OpenCms.log(A_OpenCms.C_OPENCMS_CRITICAL,
						"IndexManager: " +
						"CMS(2) Error processing file " + dir + ": " +
						cmse.toString());
				}
				++failedFiles;
			}
		}
		return;
	}

	/**
	 * Convenience method for processFile(CmsFile, IndexWriter).
	 * 
	 */ 
	protected void processFile(String file, IndexWriter writer, String section) 
		throws CmsException, java.io.IOException
	{
		processFile(cmso.readFile(file), writer, section);
		return;
	}
	
	/**
	 * Take the appropriate DocumentFactory and apply it to the file. Then
	 * add it to the writer. NOTE: Factories that are disabled in the config
	 * will not be run, so if you disable XML Template, for instance, then 
	 * they will not get added to the index.
	 * <p>Okay... now some types have slipped through here. 
	 * Links, Binary, CompatiblePlain, Folder and Image are not checked
	 * here because there is no need. However, if one were to add a new
	 * resource type, it would not get added to the search index 
	 * without explicitely modifying/subclassing this class.</p>
	 * @param CmsFile file to be added to index.
	 * @param IndexWriter index being created/updated. 
	 */ 
	protected void processFile(CmsFile file, IndexWriter writer, String section) 
		throws CmsException, java.io.IOException 
	{
		/*
		 * Determine the type of file, and then construct the correct
		 * Document for it, and add it to the writer.
		 *
		 * Note that factories may be turned off in the config, in which
		 * case, trying to get one will return null. That is okay -- not
		 * an error and not worth reporting (considering some instances like
		 * XML Templates are usually turned off)
		 */
		int type = file.getType();
		String typeName = cmso.getResourceType(type).getResourceTypeName();
		HashMap extraParams = new HashMap(1);
		extraParams.put("section", section);
		I_DocumentFactory df;
		if(INDEX_TYPE_PAGE.equals(typeName)) {
			df = sParams.getPageDocumentFactory();
			if(df == null) return; // This factory was disabled by setup.
			writer.addDocument(df.Document(cmso, file, extraParams));
		} else if (INDEX_TYPE_PLAIN.equals(typeName)) {
			// This will use the extension mapper to figure out which
			// DocumentFactory to use.
			df = sParams.getPlainDocumentFactory(file);
			if(df == null) return;
			writer.addDocument(df.Document(cmso, file, extraParams));
		} else if (INDEX_TYPE_JSP.equals(typeName)) {
			df = sParams.getJspDocumentFactory();
			if(df == null) return;
			writer.addDocument(df.Document(cmso, file, extraParams));
		} else if (INDEX_TYPE_XML_TEMPLATE.equals(typeName)) {
			df = sParams.getXmlTemplateDocumentFactory();
			if(df == null) return; // This should usually be null.
			writer.addDocument(df.Document(cmso, file, extraParams));
		}
		return;
	}
}
