package net.grcomputing.opencms.search.lucene;

/*
* <p>
* This file is part of GRC's OpenCMS module library.
* </p>
* <p>
* GRC's OpenCMS module library 
* is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
* </p>
* <p>
* GRC's OpenCMS module library is distributed in the 
* hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* </p>
* <p>
* You should have received a copy of the GNU General Public License
* along with GRC's OpenCMS module library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
* Or view the license online at http://www.gnu.org/licenses/gpl.html
* </p>
* <hr>
*/
import java.util.Date;
import java.util.HashMap;

import org.apache.lucene.document.DateField;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;

import com.opencms.core.CmsException;
import com.opencms.file.CmsFile;
import com.opencms.file.CmsObject;
import com.opencms.template.CmsXmlControlFile;
import com.opencms.template.CmsXmlTemplateFile;

/**
* <p>This class serves as a document factory for OpenCMS resources. It produces
* Lucene Document objects that contain the correct fields for indexing OpenCMS
* resources. Unlike some of the other Lucene implementations, this one is
* highly coupled with the OpenCMS API - thereby taking advantage of properties
* security settings, etc.</p>
* <p>Essentially, the main purpose of this method is to provide factories for
* Document instances that are specific to the types of resources in OpenCMS.
* For instance, we know that a Page will have a name, a title, and probably
* description and keywords. We also know that it is an XML document with HTML
* stored in the CDATA section. These bits of information provide information
* that we can use to generate an appropriate Document representation.</p>
* @author Matt Butcher mbutcher@grcomputing.net
* @see <a href="http://grcomputing.net">http://grcomputing.net</a>
*/
public class PageDocument implements I_DocumentConstants, I_DocumentFactory {
	/**
	 * Takes a Page instance and builds a Lucene Document suitable for 
	 * index generation.
	 * @param CmsObject cms - instance of CmsObject
	 * @param CmsFile f - file that we will generate Document from.
	 * @param HashMap params - extra params
	 * @throws CmsException it cannot work with the CmsFile or CmsObject.
	 */
	public Document Document(CmsObject cmso, CmsFile f, HashMap p)
		throws CmsException 
	{
		/* Can't find a way to get resource type or typeName from both
		 * object CmsFile and constants.
		if(f.getTypeName() != cmso.C_TYPE_PAGE_NAME) 
			throw new CmsException("CmsFile is not a Page!");;
		*/

		Document doc = new Document();
		String absPath = f.getAbsolutePath();

		// This is location in VFS -- not URL.
		doc.add(Field.UnIndexed(FIELD_PATH, absPath));

		// Get date info. I hope the long is actually a millisecond string...
		doc.add(Field.Keyword(FIELD_INITIAL_ADD, 
			DateField.timeToString(f.getDateCreated())));

		doc.add(Field.Keyword(FIELD_LAST_MOD, 
			DateField.timeToString(f.getDateLastModified())));

		doc.add(Field.Keyword("indexed_on", 
			DateField.dateToString(new Date())));

		// Check for properties:
		String title, description, keywords;
	
		if((title = cmso.readProperty(absPath, "title")) != null) {
			doc.add(Field.Keyword(FIELD_TITLE, title));
		} else doc.add(Field.Keyword(FIELD_TITLE, "Untitled"));

		if((description = cmso.readProperty(absPath, "description")) != null) {
			doc.add(Field.Keyword(FIELD_DESC, description));
		}

		if((keywords = cmso.readProperty(absPath, "keywords")) != null) {
			doc.add(Field.Keyword(FIELD_KEYWORDS, keywords));
		}

		if(p!=null && (!(p.get("section")==null))) 
		{
			doc.add(Field.Keyword(FIELD_SECTION, (String)p.get("section")));
		}
		// Any other properties?

		/*
		 * Now we move on to the contents of the Page. What we need to do is
		 * get the body file (from CmsXmlControlFile), then get the 
		 * <TEMPLATE<![CDATA[]]></TEMPLATE> from it. That info is in HTML,
		 * so we need to parse that out and get just the text.
		 */
		CmsXmlControlFile xCntrl = new CmsXmlControlFile(cmso, f);
		String contentsName = xCntrl.getElementTemplate("body");
		CmsFile contents = cmso.readFile(contentsName);
		CmsXmlTemplateFile xcContents = new CmsXmlTemplateFile(cmso, contents);

		// We want the default body.
		// Is it not just a little funny that this works?
		String cdata = xcContents.getTemplateContent(null,null,null);
		/* Probably better for future releases is this:
		 * Note that null instead of "TEMPLATE" is better.
		String cdata = xcContents.getTemplateContent(new Object(),
			new java.util.Hashtable(), null);
		*/

		// HTMLParser p = new HTMLParser
		String cleanCdata = FastTagStripper.strip(cdata.toCharArray());

		// Need either a reader or string. CmsFile supports byteArray.
		doc.add(Field.Text(FIELD_BODY, cleanCdata ));
		
		return doc;
	}

	/**
	 * Takes a Page instance and builds a Lucene Document suitable for 
	 * index generation.
	 * @param CmsObject cms - instance of CmsObject
	 * @param CmsFile f - file that we will generate Document from.
	 * @throws CmsException it cannot work with the CmsFile or CmsObject.
	 */
	public Document Document(CmsObject cmso, CmsFile f)
		throws CmsException 
	{
		return Document(cmso, f, null);
	}
}
