package net.grcomputing.opencms.search.lucene;

/*
* <p>
* This file is part of GRC's OpenCMS module library.
* </p>
* <p>
* GRC's OpenCMS module library 
* is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
* </p>
* <p>
* GRC's OpenCMS module library is distributed in the 
* hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* </p>
* <p>
* You should have received a copy of the GNU General Public License
* along with GRC's OpenCMS module library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
* Or view the license online at http://www.gnu.org/licenses/gpl.html
* </p>
* <hr>
* 
*/

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.StopAnalyzer;
import org.apache.lucene.queryParser.QueryParser;
import org.apache.lucene.search.BooleanQuery;
import org.apache.lucene.search.Hits;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.Searcher;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.opencms.core.CmsException;
import com.opencms.file.CmsObject;
import com.opencms.file.CmsRegistry;
import com.opencms.file.I_CmsRegistry;
import com.opencms.flex.jsp.CmsJspActionElement;

/**
* Provides convenience functions for JSPs or external classes that want to
* provide Lucene search capablilites.
* @author Matt Butcher mbutcher@grcomputing.net
* @see <a href="http://grcomputing.net">http://grcomputing.net</a>
* 
*/
public class SearchHelper implements I_XmlConfigConstants {

	private CmsObject cmso = null;
	private Element ele_luceneSearch = null;

	/** 
	 * Fields that get indexed. 
	 * @see I_DocumentConstants
	 */
	protected static String[] indexedFields = {
		"body", "last_modified", "initial_add", "title","description",
		"keywords"
	};

	private SearchHelper() {}

	/**
	 * Creates new SearchHelper, getting information out of the 
	 * main registry. This is a convenience constructor for JSPs. 
	 * (The registry is usually found in 
	 * $TOMCAT_HOME/webapps/opencms/WEB_INF/config/registry.xml)
	 * @param CmsJspActionElement page action element
	 * @throws CmsException if no lucene config info can be found in registry,
	 * or if there were problems trying to work with the registry.
	 */
	public SearchHelper(CmsJspActionElement cms) throws CmsException {
		this.cmso = cms.getCmsObject();
		I_CmsRegistry reg = cmso.getRegistry();
		Element ele_sys = reg.getSystemElement();
		ele_luceneSearch = getLuceneSearchElement(ele_sys);
		if (ele_luceneSearch == null)
			throw new CmsException("No Lucene search config found in registry");
		
	} 

	/**
	 * Creates new SearchHelper, getting information out of the 
	 * main registry. (usually 
	 * $TOMCAT_HOME/webapps/opencms/WEB_INF/config/registry.xml)
	 * @param CmsObject 
	 * @throws CmsException if no lucene config info can be found in registry,
	 * or if there were problems trying to work with the registry.
	 */
	public SearchHelper(CmsObject cmsobject) throws CmsException {
		this.cmso = cmsobject;
		I_CmsRegistry reg = cmso.getRegistry();
		Element ele_sys = reg.getSystemElement();
		ele_luceneSearch = getLuceneSearchElement(ele_sys);
		if (ele_luceneSearch == null)
			throw new CmsException("No Lucene search config found in registry");
		
	} 

	/**
	 * Creates new SearchHelper, using the registry provided. (rather than the
	 * default registry).
	 * @param CmsObject 
	 * @param String path to the alternate registry.
	 * @throws CmsException if no registry is found, or 
	 * no lucene config info can be found in registry,
	 * or if there were problems trying to work with the registry.
	 */
	public SearchHelper(CmsObject cmsobject, String registryPath) 
		throws CmsException 
	{
		this.cmso = cmsobject;
		I_CmsRegistry reg = new CmsRegistry(registryPath);
		Element ele_sys = reg.getSystemElement();
		ele_luceneSearch = getLuceneSearchElement(ele_sys);
		if (ele_luceneSearch == null)
			throw new CmsException("No Lucene search config found in registry");
	} 


	/**
	 * Creates new SearchHelper, using the registry provided. (rather than the
	 * default registry).
	 * @param CmsObject 
	 * @param String path to the alternate registry.
	 * @throws CmsException if no registry is found, or 
	 * no lucene config info can be found in registry,
	 * or if there were problems trying to work with the registry.
	 */
	public SearchHelper(CmsObject cmsobject, Element luceneElement) {
		this.cmso = cmsobject;
		this.ele_luceneSearch = luceneElement;
	} 
	/**
	 * Get the path for the Lucene search index.
	 * @return path to the Lucene index.
	 * @throws CmsException if there is no index element or no data in the
	 * index element.
	 */
	public String getIndexPath() throws CmsException  {
		Element ele_index = getFirstElement(ele_luceneSearch,	
			ELE_INDEX_DIR);
		if(ele_index == null) 
			throw new CmsException("No index path element in search config");
		String indexPath =  getData(ele_index);
		if(indexPath == null) 
			throw new CmsException("No index path in search config");
		return indexPath;
		
	}

	
	/**
	 * Does a very simple search, running a StopAnalyzer against a search
	 * within bodies.
	 * @param String search string.
	 * @return Hits containing lucene Documents
	 * @throws CmsException if search fails or there is a problem getting the
	 * the information necessary for a search.
	 */
	public Hits doSimpleSearch(String term) throws CmsException
	{
		return this.doSimpleSearch(term, null);
	}
	
	
	/**
	 * Does a very simple search, running a StopAnalyzer against a search
	 * within bodies.
	 * @param String search string.
	 * @return Hits containing lucene Documents
	 * @throws CmsException if search fails or there is a problem getting the
	 * the information necessary for a search.
	 */
	public Hits doSimpleSearch(String term, String section) throws CmsException {
		try {
			Searcher s = new IndexSearcher(getIndexPath());
			Query q, sec = null;
			Hits hits = null;
			BooleanQuery combined = null;
			Analyzer analyzer = new StopAnalyzer();
			q = QueryParser.parse(term, "body", analyzer);
			if (section != null)
			{
				sec = QueryParser.parse(section, "section", analyzer);
				combined = new BooleanQuery();
				combined.add(q, true, false);
				combined.add(sec, true, false);
				hits = s.search(combined);
			}
			else
			{
				hits = s.search(q);
			}
			return hits;
		} catch (java.io.IOException ioe) {
			throw new CmsException("Error searching", ioe);	
		} catch (org.apache.lucene.queryParser.ParseException pe) {
			throw new CmsException("Error parsing search string", pe);	
		}
	}

	/**
	 * Returns a list of indexed fields. NOTE: at this time, this is actually
	 * a static array reflecting the fields I_DocumentConstants defines.
	 * This will probably change in the future.
	 * @return an array of Strings, each naming a particular field that was
	 * indexed. 
	 * @see I_DocumentConstants
	 */
	public String[] getIndexedFields() {
		return indexedFields;
	}

/* ========================================================================= */

	/**
	 * Gets the right node from the system element. Returns null if there
	 * is none.
	 * @param Element sys - system element.
	 */
	private static Element getLuceneSearchElement(Element sys) {
		NodeList nl = sys.getElementsByTagName(LUCENE_SEARCH_ELEMENT);
		if(nl.getLength() > 0) return (Element)nl.item(0);
		return null;
	}

	/**
	 * Returns text or CDATA content from a node. This catenates all of the
	 * text and cdata nodes in an element, so for example two text nodes and a
	 * CDATA section will all be combined into one String.
	 */
	private static String getData(Element ele_r) {
		NodeList nl = ele_r.getChildNodes();
		Node node_tmp;
		StringBuffer sb = new StringBuffer();
		int i, j= nl.getLength();
		for (i = 0; i < j; ++i) {
			node_tmp = nl.item(i);
			short nType = node_tmp.getNodeType();	
			if(nType == node_tmp.TEXT_NODE || 
				nType == node_tmp.CDATA_SECTION_NODE) 
			{
				sb.append(node_tmp.getNodeValue());
			}
		}
		if(sb.length() == 0) return null;
		return sb.toString();
	}

	private Element getFirstElement(Element ele_r, String name) {
		NodeList nl = ele_r.getElementsByTagName(name);
		if(nl.getLength() == 0) return null;
		Element ele_f = (Element)nl.item(0);
		return ele_f;
	}


}


