package net.grcomputing.opencms.search.lucene;

/*
* <p>
* This file is part of GRC's OpenCMS module library.
* </p>
* <p>
* GRC's OpenCMS module library 
* is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
* </p>
* <p>
* GRC's OpenCMS module library is distributed in the 
* hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* </p>
* <p>
* You should have received a copy of the GNU General Public License
* along with GRC's OpenCMS module library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
* Or view the license online at http://www.gnu.org/licenses/gpl.html
* </p>
* <hr>
* 
*/

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.opencms.core.CmsException;
import com.opencms.file.CmsFile;
import com.opencms.file.CmsObject;
import com.opencms.file.CmsRegistry;
import com.opencms.file.I_CmsRegistry;

/**
* <p>This class retrieves/sets parameters for searching.</p>
* @author Matt Butcher mbutcher@grcomputing.net
* @see <a href="http://grcomputing.net">http://grcomputing.net</a>
*/
public class SearchParameters implements I_XmlConfigConstants {

	/* This is the main element we are concerned with. */
	private Element ele_luceneSearch; 

	private boolean subsearch = false;
	private String indexPath = "";
	private String analyzer = 
		"org.apache.lucene.analysis.standard.StandardAnalyzer";
	private boolean securityCheck = true;
	private int mergeFactor = 100000;
	private String projectName = "online";

	private HashMap dirs = new HashMap();

	private I_DocumentFactory pageDocumentFactory;
	private boolean pageDocFacEnabled = false;

	private I_DocumentFactory jspDocumentFactory;
	private boolean jspDocFacEnabled = false;

	private I_DocumentFactory xmlTempDocumentFactory;
	private boolean xmlTempDocFacEnabled = false;

	private ArrayList extensionMappings;
	private boolean plainDocFacEnabled = false;

	private SearchParameters() {};

	/**
	 * This method constructs a new SearchParameters by retrieving 
	 * configuration data from the current system registry. The
	 * registry is usually located somewhere like 
	 * $TOMCAT/webapps/opencms/WEB-INF/config/registry.xml
	 * @param CmsObject cmso - used to get the registry.
	 * @throws CmsException if it can't get the registry or there is no search
	 * config present in the registry.
	 */
	public SearchParameters(CmsObject cmso) throws CmsException {
		Element ele_sys = cmso.getRegistry().getSystemElement();
		ele_luceneSearch = getLuceneSearchElement(ele_sys);
		if (ele_luceneSearch == null)
			throw new CmsException("No Lucene search config found in registry");
		readConfig();
		return;
	}

	/**
	 * Constructs a new SearchParameters by getting a registry from the 
	 * main file system (not VFS).
	 * @param String registryPath - full system path for registry info (e.g.
	 * /opt/tomcat/webapps/opencms/WEB-INF/config/myRegistry.xml)
	 * @throws CmsException if it can't get the registry or there is no search
	 * config present in the registry.
	 */
	public SearchParameters(String registryPath) throws CmsException {
		I_CmsRegistry reg = new CmsRegistry(registryPath);
		Element ele_sys = reg.getSystemElement();
		ele_luceneSearch = getLuceneSearchElement(ele_sys);
		if (ele_luceneSearch == null)
			throw new CmsException("No Lucene search config found in registry");
		readConfig();
		return;
	}

	/**
	 * Constructs a SearchParameters from an org.w3c.dom.Element. The element
	 * is assumed to be the equivalent of the luceneSearch element that is
	 * fetched from the registry, but it need not be named the same thing.
	 */
	public SearchParameters(Element ele_config) throws CmsException {
		this.ele_luceneSearch = ele_config;
		readConfig();
	}

	/**
	 * Returns true if the DocumentFactory is enabled for this type.
	 */
	public boolean isUsingPageDocumentFactory() {
		return pageDocFacEnabled;
	}

	/**
	 * Returns true if the DocumentFactory is enabled for this type.
	 */
	public boolean isUsingPlainDocumentFactory() {
		return plainDocFacEnabled;

	}

	/**
	 * Returns true if the DocumentFactory is enabled for this type.
	 */
	public boolean isUsingJspDocumentFactory() {
		return jspDocFacEnabled;
	}

	/**
	 * Returns true if the DocumentFactory is enabled for this type.
	 */
	public boolean isUsingXmlTemplateDocumentFactory() {
		return xmlTempDocFacEnabled;
	}

	/** 
	 * Get the value of global subsearching. If true, then a subtree search
	 * should be run on all directories which do not specify otherwise.
	 */
	public boolean needsSubsearch() {
		return subsearch;
	}

	/**
	 * Returns true if global viewing permissions should be checked before 
	 * making the index.
	 * If true, then file permissions will be checked before indexing.
	 */
	public boolean needsSecurityCheck() {
		return securityCheck;
	}

	/**
	 * returns the mergeFactor for the indexer.
	 */
	public int getMergeFactor() {
		return mergeFactor;
	}

	/**
	 * Returns the project that the indexer ought to index against.
	 * @return project name
	 */
	public String getProject() {
		return projectName;
	}

	/**
	 * Returns the classname of the appropropriate analyzer.
	 */
	public String getAnalyzerName() {
		return analyzer;
	}

	/**
	 * Returns the path that Lucene should use for creating indexes. Path
	 * is on real filesystem, not VFS.
	 */
	public String getIndexPath() {
		return indexPath;
	}

	/**
	 * Returns HashMap of (String dirPath, String enabled). The second string
	 * should be either true or false, but it may be "" or null as well. It is
	 * not checked when it is read in from the filesystem.
	 * @return HashMap of (String path, String enabled)
	 */
	public HashMap getDirectories() {
		return dirs;
	}

	/**
	 * returns an instance of a page-specific DocumentFactory.
	 * @return DocumentFactory instance or null if disabled.
	 */
	public I_DocumentFactory getPageDocumentFactory() {
		if(pageDocFacEnabled) return pageDocumentFactory;
		return null;
	}

	/**
	 * returns an instance of a jsp-specific DocumentFactory.
	 * @return DocumentFactory instance or null if disabled.
	 */
	public I_DocumentFactory getJspDocumentFactory() {
		if(jspDocFacEnabled) return jspDocumentFactory;
		return null;
	}

	/**
	 * returns an instance of an XMLTemplate-specific DocumentFactory.
	 * @return DocumentFactory instance or null if disabled.
	 */
	public I_DocumentFactory getXmlTemplateDocumentFactory() {
		if(xmlTempDocFacEnabled) return xmlTempDocumentFactory;
		return null;
	}

	/**
	 *	Using CmsFile, return the appropriate DocumentFactory. This uses
	 * extension mapping to guess the best sort of DocumentFactory for
	 * indexing this file.
	 * <p>If the extension is not handled by any ExtensionMaps, then the
	 * file ought to be left out of the search index (or you ought to 
	 * fix your config files/write a new DocumentFactory)</p>
	 * @param CmsFile file to index
	 * @throws CmsException if it can't get the right class loaded.
	 */
	public I_DocumentFactory getPlainDocumentFactory(CmsFile f) 
		throws CmsException
	{
		if(plainDocFacEnabled) {
			String ext = f.getExtension();
			return getPlainDocumentFactory(ext);
		}
		return null;
	}

	/**
	 *	Using String extension, return the appropriate DocumentFactory. 
	 * This uses
	 * extension mapping to guess the best sort of DocumentFactory for
	 * indexing this file. Extension should be of the form ".xxx"
	 * <p>If the extension is not handled by any ExtensionMaps, then the
	 * file ought to be left out of the search index (or you ought to 
	 * fix your config files/write a new DocumentFactory)</p>
	 * @param CmsFile file to index
	 * @throws CmsException if it can't get the right class loaded.
	 */
	public I_DocumentFactory getPlainDocumentFactory(String extension)
		throws CmsException
	{
		/*
		 * It is always possible that the file either doesn't have an 
		 * extension, or has a single dot for an extension (which is 
		 * not good). A valid extension has at least one dot and one
		 * other character. Since the ExtensionMapping does more robust
		 * testing (and correcting), this is all we need to do.
		 */
		if(extension == null || extension.length() < 2) return null;
		if(extensionMappings == null)
			throw new CmsException("Extension Mappings were not initialized!");

		ExtensionMapping map;
		Iterator i = extensionMappings.iterator();
		while(i.hasNext()) {
			map = (ExtensionMapping)i.next();
			if(map == null) 
				throw new CmsException("Null ExtensionMap instance");
			if(map.handlesExtension(extension)) return map.getDocumentFactory();
		}
		return null;
	}

	/**
	 * Get the extension mappings for Plain DocumentFactories. Plain-type
	 * documents in OpenCMS represent all kinds of file types, each of which
	 * may need a different type of indexer. To work around this, there is a
	 * separate method of building Page DocumentFactories. 
	 * <p>The ArrayList returned contains ExtensionMapping objects, which map
	 * file extensions to the appropriate DocumentFactory. Use these mappings
	 * to get the correct DocumentFactory for a plain-type document.</p>
	 * <p>If none of this makes sense to yoou, use the getPlainDocumentFactory
	 * method.</p>
	 * @see net.grcomputing.opencms.search.lucene.ExtensionMapping
	 */
	public ArrayList getPlainExtensionMappings() {
		return extensionMappings;
	}

/*=============================================================================
	NON-PUBLIC METHODS
=============================================================================*/

	/**
	 * Gets the right node from the system element. Returns null if there
	 * is none.
	 * @param Element sys - system element.
	 */
	protected Element getLuceneSearchElement(Element sys) {
		NodeList nl = sys.getElementsByTagName(LUCENE_SEARCH_ELEMENT);
		if(nl.getLength() > 0) return (Element)nl.item(0);
		return null;
	}

	/** Reads the config information and stores it. */
	protected void readConfig() throws CmsException {
		// subsearch:
		Element ele_t = getFirstElement(ele_luceneSearch, ELE_SUBSEARCH);
		if(ele_t != null && "true".equals(getData(ele_t))) subsearch = true;
		else subsearch = false;

		//indexPath
		ele_t = getFirstElement(ele_luceneSearch, ELE_INDEX_DIR);
		if(ele_t != null) {
			String tPath = getData(ele_t);
			if(null != tPath) indexPath = tPath;
		}

		//securityCheck
		ele_t = getFirstElement(ele_luceneSearch, ELE_PERM_CHECK);
		if(ele_t != null && "true".equals(getData(ele_t))) securityCheck = true;
		else securityCheck = false;

		// analyzer
		ele_t = getFirstElement(ele_luceneSearch, ELE_ANALYZER);
		if(ele_t != null) {
			String tAna = getData(ele_t);
			if(tAna != null) analyzer = tAna;
		}

		// project
		ele_t = getFirstElement(ele_luceneSearch, ELE_PROJECT);
		if(ele_t != null) {
			String tProj = getData(ele_t);
			if(tProj != null) projectName = tProj;
		}

		// mergeFactor
		ele_t = getFirstElement(ele_luceneSearch, ELE_MERGE_FACTOR);
		if(ele_t != null) {
			String tMF = getData(ele_t);
			try {
				mergeFactor = Integer.parseInt(tMF);
			} catch (NumberFormatException nfe) {
				throw new CmsException(
					"Error getting Merge Factor from config",
					nfe);
			}
		}

		ele_t = getFirstElement(ele_luceneSearch, ELE_DOC_FACTORIES);

		/*
		 * Configure Page Document Factory.
		 */
		//Element ele_pdf = getFirstElement(ele_luceneSearch, 
		Element ele_pdf = getFirstElement(ele_t, 
			ELE_PAGE_DOC_FACTORY);
		/*
		 * Initially I threw exceptions for missing elements, but I've
		 * decided that it is the admin's prerogative to not configure a
		 * document factory, so a missing config is considered a disabled
		 * doc factory.
		 */
		/*
		if(ele_pdf == null) throw new CmsException("Can't find "+
			ELE_PAGE_DOC_FACTORY + " element";
		*/

		if(ele_pdf != null && "true".equals(ele_pdf.getAttribute(ATTR_ENABLED)))
		{
			Element ele_class = getFirstElement(ele_pdf, ELE_CLASS);
			String cname = getData(ele_class);
			if(cname != null) {
				try {
					Class lx = Class.forName(cname);
					pageDocumentFactory = (I_DocumentFactory)lx.newInstance();
					pageDocFacEnabled = true;
				} catch (ClassNotFoundException cnfe) {
					throw new CmsException("No pageDocumentFactory class "
						+"could be found that matched "+cname, cnfe);
				} catch (InstantiationException ie) {
					throw new CmsException("Could not create instance of "
						+"pageDocumentFactory class "+cname, ie);
				} catch (IllegalAccessException iae) {
					throw new CmsException("Illegal access of "+cname, iae);
				}
			} else
				throw new CmsException("Configuration error: " +
					"No <class> set for pageDocumentFactory.");
		} else {
			pageDocFacEnabled = false;
		}
		
		/*
		 * Configure Plain Document Factory.
		 * Plain documents are much more difficult to do than other documents.
		 * For the time being, this is going to be pretty hacked up -- but 
		 * the result is a better way to handle plain documents.
		 */
		ele_pdf = getFirstElement(ele_t,
			ELE_PLAIN_DOC_FACTORY);
		/*
		if(ele_pdf == null) throw new CmsException("Can't find "+
			ELE_PLAIN_DOC_FACTORY + " element";
		*/
		if(ele_pdf != null && "true".equals(ele_pdf.getAttribute(ATTR_ENABLED)))
		{
			extensionMappings = getExtensionMappings(ele_pdf);

			if(extensionMappings.size() == 0)
				plainDocFacEnabled = false;
			else
				plainDocFacEnabled = true;

		} else {
			plainDocFacEnabled = false;
		}

		/*
		 * Configure JSP Document Factory.
		 */
		ele_pdf = getFirstElement(ele_t, 
			ELE_JSP_DOC_FACTORY);
		if(ele_pdf != null && "true".equals(ele_pdf.getAttribute(ATTR_ENABLED)))
		{
			Element ele_class = getFirstElement(ele_pdf, ELE_CLASS);
			String cname = getData(ele_class);
			if(cname != null) {
				try {
					Class lx = Class.forName(cname);
					jspDocumentFactory = (I_DocumentFactory)lx.newInstance();
					jspDocFacEnabled = true;
				} catch (ClassNotFoundException cnfe) {
					throw new CmsException("No jspDocumentFactory class "
						+"could be found that matched "+cname, cnfe);
				} catch (InstantiationException ie) {
					throw new CmsException("Could not create instance of "
						+"jspDocumentFactory class "+cname, ie);
				} catch (IllegalAccessException iae) {
					throw new CmsException("Illegal access of "+cname, iae);
				}
			} else
				throw new CmsException("Configuration error: " +
					"No <class> set for jspDocumentFactory.");
		} else {
			jspDocFacEnabled = false;
		}


		/*
		 * Configure XML Template Document Factory.
		 */
		ele_pdf = getFirstElement(ele_t, 
			ELE_XML_TEMPLATE_DOC_FACTORY);
		if(ele_pdf != null && "true".equals(ele_pdf.getAttribute(ATTR_ENABLED)))
		{
			Element ele_class = getFirstElement(ele_pdf, ELE_CLASS);
			String cname = getData(ele_class);
			if(cname != null) {
				try {
					Class lx = Class.forName(cname);
					xmlTempDocumentFactory = 
						(I_DocumentFactory)lx.newInstance();
					xmlTempDocFacEnabled = true;
				} catch (ClassNotFoundException cnfe) {
					throw new CmsException("No xmlTemplateDocumentFactory class "
						+"could be found that matched "+cname, cnfe);
				} catch (InstantiationException ie) {
					throw new CmsException("Could not create instance of "
						+"xmlTemplateDocumentFactory class "+cname, ie);
				} catch (IllegalAccessException iae) {
					throw new CmsException("Illegal access of "+cname, iae);
				}
			} else
				throw new CmsException("Configuration error: " +
					"No <class> set for pageDocumentFactory.");
		} else {
			xmlTempDocFacEnabled = false;
		}

		ele_t = getFirstElement(ele_luceneSearch, ELE_DIRECTORIES);
		/*
		 * Well, this would be an interesting scenario... what if 
		 * no directories were specified? I guess we would just
		 * quite here. It doesn't make sense to throw an exception, as
		 * this would be a likely candidate for quickly turning off search
		 * indexing. On the other hand... it seems rather odd to just
		 * ignore the fact that the search engine is configured, but there
		 * is nothing to search.
		 */
		if(ele_t == null) return; 

		NodeList nl_dirs = ele_t.getElementsByTagName(ELE_DIRECTORY);
		Element ele_dir, ele_sub, ele_sect;
		String path, dirSubsearch, section = null;
		Map dirValues = null;
		int p, q = nl_dirs.getLength();
		for (p = 0; p < q; ++p) {
			ele_dir = (Element)nl_dirs.item(p);
			ele_sub =  getFirstElement(ele_dir, ELE_SUBSEARCH);
			ele_sect = getFirstElement(ele_dir, ELE_SECTION);
			path = ele_dir.getAttribute(ATTR_LOCATION);
			dirSubsearch = getData(ele_sub);
			if (ele_sect != null)
				section = getData(ele_sect);
			if(path != null && !"".equals(path))
			{
			  dirValues = new HashMap(3);
			  dirValues.put(ATTR_LOCATION, path);
				dirValues.put(ELE_SUBSEARCH, dirSubsearch);
				dirValues.put(ELE_SECTION, section);
				dirs.put(path, dirValues);
			}
		}
		return;

	}

	private Element getFirstElement(Element ele_r, String name) {
		NodeList nl = ele_r.getElementsByTagName(name);
		if(nl.getLength() == 0) return null;
		Element ele_f = (Element)nl.item(0);
		return ele_f;
	}

	/**
	 * Returns text or CDATA content from a node. This catenates all of the
	 * text and cdata nodes in an element, so for example two text nodes and a
	 * CDATA section will all be combined into one String.
	 */
	private String getData(Element ele_r) {
		NodeList nl = ele_r.getChildNodes();
		Node node_tmp;
		StringBuffer sb = new StringBuffer();
		int i, j= nl.getLength();
		for (i = 0; i < j; ++i) {
			node_tmp = nl.item(i);
			short nType = node_tmp.getNodeType();	
			if(nType == node_tmp.TEXT_NODE || 
				nType == node_tmp.CDATA_SECTION_NODE) 
			{
				sb.append(node_tmp.getNodeValue());
			}
		}
		if(sb.length() == 0) return null;
		return sb.toString();
	}

	/**
	 * Builds ExtensionMappings for the plainDocumentFactory.
	 *
	 */
	private ArrayList getExtensionMappings(Element ele_plain) 
		throws CmsException
	 {
		NodeList nl = ele_plain.getElementsByTagName(ELE_FILE_TYPE);
		ArrayList mappings = new ArrayList();

		/*
		 * Get a list of fileTypes.
		 */
		Element ele_t, ele_ext, ele_class;
		String extMapName, extMapClass, extText;
		NodeList nl_ext;
		ExtensionMapping extMap;
		int i, j = nl.getLength();
		for (i=0; i<j; ++i) { 
			ele_t = (Element)nl.item(i);
			extMapName = ele_t.getAttribute(ATTR_NAME);
			if(extMapName == null || "".equals(extMapName))
				extMapName = "Unknown";
			ele_class = getFirstElement(ele_t, ELE_CLASS);
			extMapClass = getData(ele_class);
			if (extMapClass == null || "".equals(extMapClass))
				throw new CmsException("Missing class for extensions with "
					+ "name "+extMapName);

			/*
			 * Create a new ExtensionMapping and add extensions.
			 */
			extMap = new ExtensionMapping(extMapClass, extMapName);
			nl_ext = ele_t.getElementsByTagName(ELE_EXTENSION);
			int k, l = nl_ext.getLength();
			for (k = 0; k < l; ++k) {
				ele_ext = (Element)nl_ext.item(k);
				extText = getData(ele_ext);
				if(extText != null && !"".equals(extText)) 
					extMap.addExtension(extText);
			}

			// Add the new ExtensionMapping to the collection.
			mappings.add(extMap);
		}
		return mappings;
	}

}




