package net.grcomputing.opencms.search.lucene;

/*
* <p>
* This file is part of GRC's OpenCMS module library.
* </p>
* <p>
* GRC's OpenCMS module library 
* is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
* </p>
* <p>
* GRC's OpenCMS module library is distributed in the 
* hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* </p>
* <p>
* You should have received a copy of the GNU General Public License
* along with GRC's OpenCMS module library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
* Or view the license online at http://www.gnu.org/licenses/gpl.html
* </p>
* <hr>
*/
import java.util.HashMap;

import org.apache.lucene.document.DateField;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;

import com.opencms.core.CmsException;
import com.opencms.file.CmsFile;
import com.opencms.file.CmsObject;

/**
* <p>This class serves as a document factory for OpenCMS resources. It produces
* Lucene Document objects that contain the correct fields for indexing OpenCMS
* resources. Unlike some of the other Lucene implementations, this one is
* highly coupled with the OpenCMS API - thereby taking advantage of properties
* security settings, etc.</p>
* <p>This class handles pages with tagged data, e.g. XML and HTML and their
* derrivatives.</p>
* @author Matt Butcher mbutcher@grcomputing.net
* @see <a href="http://grcomputing.net">http://grcomputing.net</a>
*/
public class TaggedPlainDocument 
	implements I_DocumentConstants, I_DocumentFactory 
{

	/**
	 * Takes a Plain instance of tagged content and builds a 
	 * Lucene Document suitable for 
	 * index generation. This is for Plain documents that contain tagged
	 * data -- that is, HTML, XML, and their derivatives. Like the default
	 * PageDocument parser, this one uses the fast tag stripper, which will
	 * simply strip out all of the tags from a document. Information stored
	 * in element attributes will not make it into the index. All
	 * parsed character DATA <i>will</i> make it in, even if it is JavaScript 
	 * or CSS.
	 * @see FastTagStripper
	 * @param CmsObject cms - instance of CmsObject
	 * @param CmsFile f - file that we will generate Document from.
	 * @param HashMap params - contains additional params
	 * @throws CmsException it cannot work with the CmsFile or CmsObject.
	 */
	public Document Document(CmsObject cmso, CmsFile f, HashMap p)
		throws CmsException 
	{
		Document doc = new Document();
		String absPath = f.getAbsolutePath();

		// This is location in VFS -- not URL.
		doc.add(Field.UnIndexed(FIELD_PATH, absPath));

		// Get date info. I hope the long is actually a millisecond string...
		doc.add(Field.Keyword(FIELD_INITIAL_ADD, 
			DateField.timeToString(f.getDateCreated())));

		doc.add(Field.Keyword(FIELD_LAST_MOD, 
			DateField.timeToString(f.getDateLastModified())));

		// Check for properties:
		String title, description, keywords;
		
		if((title = cmso.readProperty(absPath, "title")) != null) {
			doc.add(Field.Keyword(FIELD_TITLE, title));
		}

		if((description = cmso.readProperty(absPath, "description")) != null) {
			doc.add(Field.Keyword(FIELD_DESC, description));
		}
		if((keywords = cmso.readProperty(absPath, "keywords")) != null) {
			doc.add(Field.Keyword(FIELD_KEYWORDS, keywords));
		}
		if(p!=null && (!(p.get("section")==null))) 
		{
			doc.add(Field.Keyword(FIELD_SECTION, (String)p.get("section")));
		}

		String byteContent = new String(f.getContents());
		String pContent = FastTagStripper.strip(byteContent.toCharArray());
		doc.add(Field.Text(FIELD_BODY, pContent));
		
		return doc;

	}

	
	/**
	 * Takes a tagged Plain instance and builds a Lucene Document suitable for 
	 * index generation. Convenience method.
	 *
	 * @param CmsObject cms - instance of CmsObject
	 * @param CmsFile f - file that we will generate Document from.
	 * @throws CmsException it cannot work with the CmsFile or CmsObject.
	 */
	public Document Document(CmsObject cmso, CmsFile f)
		throws CmsException 
	{
		return Document(cmso,f,null);
	}
}
